module Splicer
  module Dynect

    # @author Matthew A. Johnston <warmwaffles@gmail.com>
    class Client
      attr_reader :base_url, :headers, :customer, :username, :password

      # @param [String] customer
      # @param [String] username
      # @param [String] password
      def initialize(customer, username, password)
        @customer = customer
        @username = username
        @password = password

        @base_url = 'https://api-v4.dynect.net/REST'
        @headers  = {
          'Content-Type' => 'application/json',
          'API-Version'  => '3.4.0'
        }
      end

      # @param [String] resource
      # @param [Hash] payload
      #
      # @return [Hash]
      def post(resource, payload={})
        execute({
          method: :post,
          url: resource_url(resource),
          payload: payload.to_json,
          headers: @headers,
          max_redirects: 20
        })
      end

      # @param [String] resource
      #
      # @return [Hash]
      def get(resource)
        execute({
          method: :get,
          url: resource_url(resource),
          headers: @headers,
          max_redirects: 20
        })
      end

      # @param [String] resource
      # @param [Hash] payload
      #
      # @return [Hash]
      def put(resource, payload={})
        execute({
          method: :put,
          url: resource_url(resource),
          payload: payload.to_json,
          headers: @headers
        })
      end

      # @param [String] resource
      #
      # @return [Hash]
      def delete(resource)
        execute({
          method: :delete,
          url: resource_url(resource),
          headers: @headers
        })
      end

      # Creates a client's session
      #
      # @return [Boolean]
      def login
        return true if logged_in?

        resp = post('Session', {
          user_name: @username,
          password: @password,
          customer_name: @customer
        })
        @headers['Auth-Token'] = resp['data']['token']
        true
      end

      # Keeps the session alive
      #
      # @return [Boolean] True if successful
      def noop
        return false unless logged_in?
        put('Session')
        true
      end

      # Destroys the client's session
      #
      # @return [Boolean]
      def logout
        return false unless logged_in?
        delete('Session')
        @headers.delete('Auth-Token')
        true
      end

      private

      # @return [Boolean]
      def logged_in?
        !!@headers['Auth-Token']
      end

      # Wrapper around RestClient::Request.execute method
      #
      # @param [Hash] args
      # @raise [Splicer::Errors::RequestError] when the request fails
      # @return [Hash]
      def execute(args={})
        response = RestClient::Request.execute(args)
        Splicer.logger.debug "[SPLICER][DYNECT] method=#{args[:method]} url=#{args[:url]} response=#{response}"
        JSON.parse(response)
      rescue RestClient::Exception => error
        Splicer.logger.debug "[SPLICER][DYNECT] method=#{args[:method]} url=#{args[:url]} response=#{error.response} message=#{error.message}"
        raise Splicer::Errors::RequestError.new(error)
      end

      def resource_url resource
        [@base_url, resource.gsub(/^\//,'').gsub(/REST\//,'')].join('/')
      end

    end

  end
end
